#include <stdlib.h>
#include <string.h>
#include <filter.h>
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <jmp-config.h>
#include <cls.h>

/* The filter string for class names */
static char* classfilter;
static size_t len;

/* One of FILTER_MATCH_ALL, FILTER_EXACT, FILTER_PACKAGE or FILTER_RECURSIVE */
static int filter_mode = FILTER_MATCH_ALL;

int filter_class_name (const char *name)
{
    int r;

    if (filter_mode == FILTER_MATCH_ALL || classfilter == NULL)
	return 1;

    if (filter_mode == FILTER_MATCH_CLASS)
	return !strcmp (name, classfilter);

    if (filter_mode == FILTER_MATCH_RECURSIVE)
	return !strncmp(name, classfilter, len);

    /* FILTER_MATCH_PACKAGE */
    r = strncmp (name, classfilter, len);
    if (r)
	return 0;
    else
	return (!strchr (name + len, '.'));
  
}

void set_filtered (cls* c) {
    cls_set_filtered (c, filter_class_name (cls_get_name (c)));
}

/** Update each class filtered variable. */
static void update_classes () {
    hashtab* classes = get_classes ();
    jmphash_lock (classes);
    jmphash_for_each ((jmphash_iter_f)set_filtered, classes);
    jmphash_unlock (classes);
}

/* Set the filter. 
 * @param filtertype one of FILTER_MATCH_*
 * @param filter the string to filter on ("rabbit.proxy").
 */
void filter_set_filter (int filtertype, const char *filter) {
    if (filter == NULL)
	return;
    filter_mode = filtertype;
    if (classfilter)
	free (classfilter);
    classfilter = strdup (filter);
    len = strlen (classfilter);
    update_classes ();
}

char* get_filter () {
    return classfilter;
}

int get_filter_type () {
    return filter_mode;
}

/* Emacs Local Variables: */
/* Emacs mode:C */
/* Emacs c-indentation-style:"gnu" */
/* Emacs c-hanging-braces-alist:((brace-list-open)(brace-entry-open)(defun-open after)(substatement-open after)(block-close . c-snug-do-while)(extern-lang-open after)) */
/* Emacs c-cleanup-list:(brace-else-brace brace-elseif-brace space-before-funcall) */
/* Emacs c-basic-offset:4 */
/* Emacs End: */
